@@ -21,7 +21,8 @@
 #include "config.h"
 #include "util.h"
 #include "errno.h"
-
+#include "devmapper.h"
+#include "strbuf.h"
 
 /*
  * significant parts of this file were taken from iscsi-bindings.c of the
@@ -60,31 +61,23 @@ valid_alias(const char *alias)
 	return 1;
 }
 
-
-static int
-format_devname(char *name, int id, int len, const char *prefix)
+static int format_devname(struct strbuf *buf, int id)
 {
-	int pos;
-	int prefix_len = strlen(prefix);
+	/*
+	 * We need: 7 chars for 32bit integers, 14 chars for 64bit integers
+	 */
+	char devname[2 * sizeof(int)];
+	int pos = sizeof(devname) - 1, rc;
 
-	if (len <= prefix_len + 1 || id <= 0)
+	if (id <= 0)
 		return -1;
 
-	memset(name, 0, len);
-	strcpy(name, prefix);
-	name[len - 1] = '\0';
-	for (pos = len - 2; pos >= prefix_len; pos--) {
-		id--;
-		name[pos] = 'a' + id % 26;
-		if (id < 26)
-			break;
-		id /= 26;
-	}
-	if (pos < prefix_len)
-		return -1;
+	devname[pos] = '\0';
+	for (; id >= 1; id /= 26)
+		devname[--pos] = 'a' + --id % 26;
 
-	memmove(name + prefix_len, name + pos, len - pos);
-	return (prefix_len + len - pos - 1);
+	rc = append_strbuf_str(buf, devname + pos);
+	return rc >= 0 ? rc : -1;
 }
 
 static int
@@ -119,6 +112,31 @@ scan_devname(const char *alias, const char *prefix)
 	return n;
 }
 
+static int
+id_already_taken(int id, const char *prefix, const char *map_wwid)
+{
+	STRBUF_ON_STACK(buf);
+	const char *alias;
+
+	if (append_strbuf_str(&buf, prefix) < 0 ||
+	    format_devname(&buf, id) < 0)
+		return 0;
+
+	alias = get_strbuf_str(&buf);
+	if (dm_map_present(alias)) {
+		char wwid[WWID_SIZE];
+
+		/* If both the name and the wwid match, then it's fine.*/
+		if (dm_get_uuid(alias, wwid, sizeof(wwid)) == 0 &&
+		    strncmp(map_wwid, wwid, sizeof(wwid)) == 0)
+			return 0;
+		condlog(3, "%s: alias '%s' already taken, but not in bindings file. reselecting alias", map_wwid, alias);
+		return 1;
+	}
+	return 0;
+}
+
+
 /*
  * Returns: 0   if matching entry in WWIDs file found
  *         -1   if an error occurs
@@ -128,7 +146,7 @@ scan_devname(const char *alias, const char *prefix)
  */
 static int
 lookup_binding(FILE *f, const char *map_wwid, char **map_alias,
-	       const char *prefix)
+	       const char *prefix, int check_if_taken)
 {
 	char buf[LINE_MAX];
 	unsigned int line_nr = 0;
@@ -183,12 +201,31 @@ lookup_binding(FILE *f, const char *map_wwid, char **map_alias,
 			return 0;
 		}
 	}
+	if (!prefix && check_if_taken)
+		id = -1;
 	if (id >= smallest_bigger_id) {
 		if (biggest_id < INT_MAX)
 			id = biggest_id + 1;
 		else
 			id = -1;
 	}
+	if (id > 0 && check_if_taken) {
+		while(id_already_taken(id, prefix, map_wwid)) {
+			if (id == INT_MAX) {
+				id = -1;
+				break;
+			}
+			id++;
+			if (id == smallest_bigger_id) {
+				if (biggest_id == INT_MAX) {
+					id = -1;
+					break;
+				}
+				if (biggest_id >= smallest_bigger_id)
+					id = biggest_id + 1;
+			}
+		}
+	}
 	if (id < 0) {
 		condlog(0, "no more available user_friendly_names");
 		return -1;
@@ -243,10 +280,10 @@ rlookup_binding(FILE *f, char *buff, const char *map_alias)
 static char *
 allocate_binding(int fd, const char *wwid, int id, const char *prefix)
 {
-	char buf[LINE_MAX];
+	STRBUF_ON_STACK(buf);
 	off_t offset;
+	ssize_t len;
 	char *alias, *c;
-	int i;
 
 	if (id <= 0) {
 		condlog(0, "%s: cannot allocate new binding for id %d",
@@ -254,16 +291,12 @@ allocate_binding(int fd, const char *wwid, int id, const char *prefix)
 		return NULL;
 	}
 
-	i = format_devname(buf, id, LINE_MAX, prefix);
-	if (i == -1)
+	if (append_strbuf_str(&buf, prefix) < 0 ||
+	    format_devname(&buf, id) == -1)
 		return NULL;
 
-	c = buf + i;
-	if (snprintf(c, LINE_MAX - i, " %s\n", wwid) >= LINE_MAX - i) {
-		condlog(1, "%s: line too long for %s\n", __func__, wwid);
+	if (print_strbuf(&buf, " %s\n", wwid) < 0)
 		return NULL;
-	}
-	buf[LINE_MAX - 1] = '\0';
 
 	offset = lseek(fd, 0, SEEK_END);
 	if (offset < 0){
@@ -271,24 +304,25 @@ allocate_binding(int fd, const char *wwid, int id, const char *prefix)
 			strerror(errno));
 		return NULL;
 	}
-	if (write(fd, buf, strlen(buf)) != (ssize_t)strlen(buf)){
+
+	len = get_strbuf_len(&buf);
+	alias = steal_strbuf_str(&buf);
+
+	if (write(fd, alias, len) != len) {
 		condlog(0, "Cannot write binding to bindings file : %s",
 			strerror(errno));
 		/* clear partial write */
 		if (ftruncate(fd, offset))
 			condlog(0, "Cannot truncate the header : %s",
 				strerror(errno));
+		free(alias);
 		return NULL;
 	}
-	c = strchr(buf, ' ');
+	c = strchr(alias, ' ');
 	if (c)
 		*c = '\0';
 
-	condlog(3, "Created new binding [%s] for WWID [%s]", buf, wwid);
-	alias = strdup(buf);
-	if (alias == NULL)
-		condlog(0, "cannot copy new alias from bindings file: out of memory");
-
+	condlog(3, "Created new binding [%s] for WWID [%s]", alias, wwid);
 	return alias;
 }
 
@@ -322,7 +356,7 @@ use_existing_alias (const char *wwid, const char *file, const char *alias_old,
 		 * allocated correctly
 		 */
 		if (strcmp(buff, wwid) == 0)
-			alias = STRDUP(alias_old);
+			alias = strdup(alias_old);
 		else {
 			alias = NULL;
 			condlog(0, "alias %s already bound to wwid %s, cannot reuse",
@@ -331,7 +365,7 @@ use_existing_alias (const char *wwid, const char *file, const char *alias_old,
 		goto out;
 	}
 
-	id = lookup_binding(f, wwid, &alias, NULL);
+	id = lookup_binding(f, wwid, &alias, NULL, 0);
 	if (alias) {
 		condlog(3, "Use existing binding [%s] for WWID [%s]",
 			alias, wwid);
@@ -388,7 +422,7 @@ get_user_friendly_alias(const char *wwid, const char *file, const char *prefix,
 		return NULL;
 	}
 
-	id = lookup_binding(f, wwid, &alias, prefix);
+	id = lookup_binding(f, wwid, &alias, prefix, 1);
 	if (id < 0) {
 		fclose(f);
 		return NULL;
@@ -518,7 +552,7 @@ static int add_binding(Bindings *bindings, const char *alias, const char *wwid)
 static int write_bindings_file(const Bindings *bindings, int fd)
 {
 	struct binding *bnd;
-	char line[LINE_MAX];
+	STRBUF_ON_STACK(line);
 	int i;
 
 	if (write(fd, BINDINGS_FILE_HEADER, sizeof(BINDINGS_FILE_HEADER) - 1)
@@ -528,16 +562,12 @@ static int write_bindings_file(const Bindings *bindings, int fd)
 	vector_foreach_slot(bindings, bnd, i) {
 		int len;
 
-		len = snprintf(line, sizeof(line), "%s %s\n",
-			       bnd->alias, bnd->wwid);
-
-		if (len < 0 || (size_t)len >= sizeof(line)) {
-			condlog(1, "%s: line overflow", __func__);
+		if ((len = print_strbuf(&line, "%s %s\n",
+					bnd->alias, bnd->wwid)) < 0)
 			return -1;
-		}
-
-		if (write(fd, line, len) != len)
+		if (write(fd, get_strbuf_str(&line), len) != len)
 			return -1;
+		truncate_strbuf(&line, 0);
 	}
 	return 0;
 }
@@ -548,13 +578,17 @@ static int fix_bindings_file(const struct config *conf,
 	int rc;
 	long fd;
 	char tempname[PATH_MAX];
+	mode_t old_umask;
 
 	if (safe_sprintf(tempname, "%s.XXXXXX", conf->bindings_file))
 		return -1;
+	/* coverity: SECURE_TEMP */
+	old_umask = umask(0077);
 	if ((fd = mkstemp(tempname)) == -1) {
 		condlog(1, "%s: mkstemp: %m", __func__);
 		return -1;
 	}
+	umask(old_umask);
 	pthread_cleanup_push(close_fd, (void*)fd);
 	rc = write_bindings_file(bindings, fd);
 	pthread_cleanup_pop(1);
