@@ -10,11 +10,11 @@
 
 #include "checkers.h"
 #include "vector.h"
-#include "memory.h"
 #include "structs.h"
 #include "util.h"
 #include "debug.h"
 #include "dmparser.h"
+#include "strbuf.h"
 
 #define WORD_SIZE 64
 
@@ -26,7 +26,7 @@ merge_words(char **dst, const char *word)
 
 	dstlen = strlen(*dst);
 	len = dstlen + strlen(word) + 2;
-	*dst = REALLOC(*dst, len);
+	*dst = realloc(*dst, len);
 
 	if (!*dst) {
 		free(p);
@@ -41,40 +41,21 @@ merge_words(char **dst, const char *word)
 	return 0;
 }
 
-#define APPEND(p, end, args...)						\
-({									\
-	int ret;							\
-									\
-	ret = snprintf(p, end - p, ##args);				\
-	if (ret < 0) {							\
-		condlog(0, "%s: conversion error", mp->alias);		\
-		goto err;						\
-	}								\
-	p += ret;							\
-	if (p >= end) {							\
-		condlog(0, "%s: params too small", mp->alias);		\
-		goto err;						\
-	}								\
-})
-
 /*
  * Transforms the path group vector into a proper device map string
  */
-int
-assemble_map (struct multipath * mp, char * params, int len)
+int assemble_map(struct multipath *mp, char **params)
 {
+	static const char no_path_retry[] = "queue_if_no_path";
+	static const char retain_hwhandler[] = "retain_attached_hw_handler";
 	int i, j;
 	int minio;
 	int nr_priority_groups, initial_pg_nr;
-	char * p;
-	const char *const end = params + len;
-	char no_path_retry[] = "queue_if_no_path";
-	char retain_hwhandler[] = "retain_attached_hw_handler";
+	STRBUF_ON_STACK(buff);
 	struct pathgroup * pgp;
 	struct path * pp;
 
 	minio = mp->minio;
-	p = params;
 
 	nr_priority_groups = VECTOR_SIZE(mp->pg);
 	initial_pg_nr = (nr_priority_groups ? mp->bestpg : 0);
@@ -87,14 +68,15 @@ assemble_map (struct multipath * mp, char * params, int len)
 	    get_linux_version_code() < KERNEL_VERSION(4, 3, 0))
 		add_feature(&mp->features, retain_hwhandler);
 
-	/* mp->features must not be NULL */
-	APPEND(p, end, "%s %s %i %i", mp->features, mp->hwhandler,
-		nr_priority_groups, initial_pg_nr);
+	if (print_strbuf(&buff, "%s %s %i %i", mp->features, mp->hwhandler,
+			 nr_priority_groups, initial_pg_nr) < 0)
+		goto err;
 
 	vector_foreach_slot (mp->pg, pgp, i) {
 		pgp = VECTOR_SLOT(mp->pg, i);
-		APPEND(p, end, " %s %i 1", mp->selector,
-		       VECTOR_SIZE(pgp->paths));
+		if (print_strbuf(&buff, " %s %i 1", mp->selector,
+				 VECTOR_SIZE(pgp->paths)) < 0)
+			goto err;
 
 		vector_foreach_slot (pgp->paths, pp, j) {
 			int tmp_minio = minio;
@@ -106,19 +88,19 @@ assemble_map (struct multipath * mp, char * params, int len)
 				condlog(0, "dev_t not set for '%s'", pp->dev);
 				goto err;
 			}
-			APPEND(p, end, " %s %d", pp->dev_t, tmp_minio);
+			if (print_strbuf(&buff, " %s %d", pp->dev_t, tmp_minio) < 0)
+				goto err;
 		}
 	}
 
-	condlog(4, "%s: assembled map [%s]", mp->alias, params);
+	*params = steal_strbuf_str(&buff);
+	condlog(4, "%s: assembled map [%s]", mp->alias, *params);
 	return 0;
 
 err:
 	return 1;
 }
 
-#undef APPEND
-
 /*
  * Caution callers: If this function encounters yet unkown path devices, it
  * adds them uninitialized to the mpp.
@@ -163,11 +145,11 @@ int disassemble_map(const struct _vector *pathvec,
 			return 1;
 
 		if (merge_words(&mpp->features, word)) {
-			FREE(word);
+			free(word);
 			return 1;
 		}
 
-		FREE(word);
+		free(word);
 	}
 
 	/*
@@ -187,10 +169,10 @@ int disassemble_map(const struct _vector *pathvec,
 			return 1;
 
 		if (merge_words(&mpp->hwhandler, word)) {
-			FREE(word);
+			free(word);
 			return 1;
 		}
-		FREE(word);
+		free(word);
 	}
 
 	/*
@@ -202,7 +184,7 @@ int disassemble_map(const struct _vector *pathvec,
 		return 1;
 
 	num_pg = atoi(word);
-	FREE(word);
+	free(word);
 
 	if (num_pg > 0) {
 		if (!mpp->pg) {
@@ -224,7 +206,7 @@ int disassemble_map(const struct _vector *pathvec,
 		goto out;
 
 	mpp->nextpg = atoi(word);
-	FREE(word);
+	free(word);
 
 	for (i = 0; i < num_pg; i++) {
 		/*
@@ -249,7 +231,7 @@ int disassemble_map(const struct _vector *pathvec,
 
 			if (merge_words(&mpp->selector, word))
 				goto out1;
-			FREE(word);
+			free(word);
 		} else {
 			p += get_word(p, NULL);
 			p += get_word(p, NULL);
@@ -277,7 +259,7 @@ int disassemble_map(const struct _vector *pathvec,
 			goto out;
 
 		num_paths = atoi(word);
-		FREE(word);
+		free(word);
 
 		p += get_word(p, &word);
 
@@ -285,7 +267,7 @@ int disassemble_map(const struct _vector *pathvec,
 			goto out;
 
 		num_paths_args = atoi(word);
-		FREE(word);
+		free(word);
 
 		for (j = 0; j < num_paths; j++) {
 			pp = NULL;
@@ -311,7 +293,7 @@ int disassemble_map(const struct _vector *pathvec,
 			} else if (store_path(pgp->paths, pp))
 				goto out1;
 
-			FREE(word);
+			free(word);
 
 			pgp->id ^= (long)pp;
 			pp->pgindex = i + 1;
@@ -320,7 +302,7 @@ int disassemble_map(const struct _vector *pathvec,
 				if (k == 0) {
 					p += get_word(p, &word);
 					def_minio = atoi(word);
-					FREE(word);
+					free(word);
 
 					if (!strncmp(mpp->selector,
 						     "round-robin", 11)) {
@@ -341,7 +323,7 @@ int disassemble_map(const struct _vector *pathvec,
 	}
 	return 0;
 out1:
-	FREE(word);
+	free(word);
 out:
 	free_pgvec(mpp->pg, KEEP_PATHS);
 	mpp->pg = NULL;
@@ -375,7 +357,7 @@ int disassemble_status(const char *params, struct multipath *mpp)
 		return 1;
 
 	num_feature_args = atoi(word);
-	FREE(word);
+	free(word);
 
 	for (i = 0; i < num_feature_args; i++) {
 		if (i == 1) {
@@ -385,7 +367,7 @@ int disassemble_status(const char *params, struct multipath *mpp)
 				return 1;
 
 			mpp->queuedio = atoi(word);
-			FREE(word);
+			free(word);
 			continue;
 		}
 		/* unknown */
@@ -400,7 +382,7 @@ int disassemble_status(const char *params, struct multipath *mpp)
 		return 1;
 
 	num_hwhandler_args = atoi(word);
-	FREE(word);
+	free(word);
 
 	for (i = 0; i < num_hwhandler_args; i++)
 		p += get_word(p, NULL);
@@ -414,7 +396,7 @@ int disassemble_status(const char *params, struct multipath *mpp)
 		return 1;
 
 	num_pg = atoi(word);
-	FREE(word);
+	free(word);
 
 	if (num_pg == 0)
 		return 0;
@@ -451,7 +433,7 @@ int disassemble_status(const char *params, struct multipath *mpp)
 			pgp->status = PGSTATE_UNDEF;
 			break;
 		}
-		FREE(word);
+		free(word);
 
 		/*
 		 * PG Status (discarded, would be '0' anyway)
@@ -464,7 +446,7 @@ int disassemble_status(const char *params, struct multipath *mpp)
 			return 1;
 
 		num_paths = atoi(word);
-		FREE(word);
+		free(word);
 
 		p += get_word(p, &word);
 
@@ -472,7 +454,7 @@ int disassemble_status(const char *params, struct multipath *mpp)
 			return 1;
 
 		num_pg_args = atoi(word);
-		FREE(word);
+		free(word);
 
 		if (VECTOR_SIZE(pgp->paths) < num_paths)
 			return 1;
@@ -502,7 +484,7 @@ int disassemble_status(const char *params, struct multipath *mpp)
 			default:
 				break;
 			}
-			FREE(word);
+			free(word);
 			/*
 			 * fail count
 			 */
@@ -512,7 +494,7 @@ int disassemble_status(const char *params, struct multipath *mpp)
 				return 1;
 
 			pp->failcount = atoi(word);
-			FREE(word);
+			free(word);
 
 			/*
 			 * selector args
@@ -525,7 +507,7 @@ int disassemble_status(const char *params, struct multipath *mpp)
 						   &def_minio) == 1 &&
 					    def_minio != mpp->minio)
 							mpp->minio = def_minio;
-					FREE(word);
+					free(word);
 				} else
 					p += get_word(p, NULL);
 			}
