@@ -194,7 +194,7 @@ static void test_dm_name_good(void **state)
 	char *name = uevent_get_dm_name(uev);
 
 	assert_string_equal(name, DM_NAME);
-	FREE(name);
+	free(name);
 }
 
 static void test_dm_name_bad_0(void **state)
@@ -205,7 +205,7 @@ static void test_dm_name_bad_0(void **state)
 	uev->envp[3] = "DM_NAME" DM_NAME;
 	name = uevent_get_dm_name(uev);
 	assert_ptr_equal(name, NULL);
-	FREE(name);
+	free(name);
 }
 
 static void test_dm_name_bad_1(void **state)
@@ -216,7 +216,7 @@ static void test_dm_name_bad_1(void **state)
 	uev->envp[3] = "DM_NAMES=" DM_NAME;
 	name = uevent_get_dm_name(uev);
 	assert_ptr_equal(name, NULL);
-	FREE(name);
+	free(name);
 }
 
 static void test_dm_name_good_1(void **state)
@@ -228,7 +228,7 @@ static void test_dm_name_good_1(void **state)
 	uev->envp[2] = "DM_NAME=" DM_NAME;
 	name = uevent_get_dm_name(uev);
 	assert_string_equal(name, DM_NAME);
-	FREE(name);
+	free(name);
 }
 
 static void test_dm_uuid_false_0(void **state)
